﻿' Add a reference to System.DirectoryServices.dll
' Add a COM reference to Active DS Type Library
Imports System.DirectoryServices
Imports ActiveDs

Public Class fmMain
    Private CurrentNode As TreeNode
    Private UserList As New ArrayList
    Private GroupList As New ArrayList

    Private Sub fmMain_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        tbMachine.Text = Environment.MachineName
        tbGroup.Text = "Administrators"
    End Sub

    Private Sub bnDoIt_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bnDoIt.Click
        Me.Cursor = Cursors.WaitCursor
        UserList.Clear()
        GroupList.Clear()
        TreeView1.Nodes.Clear()
        CurrentNode = TreeView1.Nodes.Add(tbGroup.Text)
        SearchPath("WinNT://" & tbMachine.Text & "/" & tbGroup.Text)
        TreeView1.ExpandAll()
        Me.Cursor = Cursors.Default

        MsgBox(UserList.Count & " User Accounts, and " & GroupList.Count & " Group Accounts")
    End Sub

    Private Sub SearchPath(ByVal path As String)
        Dim dirs As DirectoryEntry
        Dim members As IADsMembers
        Dim User As IADsUser
        Dim Group As IADsGroup
        Dim entry As IADs
        Dim dn, s As String

        dirs = New DirectoryEntry(path & ",group")

        members = DirectCast(dirs.Invoke("Members"), IADsMembers)
        ' let's do groups first
        For Each entry In members
            If entry.Class = "Group" Then
                Group = CType(entry, ActiveDs.IADsGroup)
                If Not GroupList.Contains(Group.Name) Then
                    GroupList.Add(Group.Name)
                End If

                If Group.Description <> "" Then
                    s = "Group: " & Group.Name & " (" & Group.Description & ")"
                Else
                    s = "Group: " & Group.Name
                End If
                If cbDN.Checked Then
                    dn = GetDN(Group.ADsPath, "Group")
                    If dn <> "" Then
                        s = s & " [" & dn & "]"
                    End If
                End If
                CurrentNode = CurrentNode.Nodes.Add(s)
                ' descend the directory structure
                SearchPath(Group.ADsPath)
                CurrentNode = CurrentNode.Parent
            End If
        Next
        ' now let's do users
        For Each entry In members
            If entry.Class = "User" Then
                User = CType(entry, ActiveDs.IADsUser)
                If Not UserList.Contains(User.Name) Then
                    UserList.Add(User.Name)
                End If
                If Not User.Name.StartsWith("S-1-") AndAlso User.FullName <> "" Then
                    s = "User: " & User.Name & " (" & User.FullName & ")"
                Else
                    s = "User: " & User.Name
                End If
                If Not User.Name.StartsWith("S-1-") AndAlso cbDN.Checked Then
                    dn = GetDN(User.ADsPath, "User")
                    If dn <> "" Then
                        s = s & " [" & dn & "]"
                    End If
                End If
                CurrentNode.Nodes.Add(s)
            End If
        Next
    End Sub

    ' return the Active Directory Distiguished Name (DN)
    Function GetDN(ByVal ADsPath As String, ByVal AD_Type As String) As String
        Static ldap As String
        Static entry As DirectoryEntry
        Static srch As DirectorySearcher
        Dim result As SearchResult
        Dim Domain, Name As String

        Domain = ADsPath.Split("/"c)(2)
        Name = ADsPath.Split("/"c)(3)

        If ldap <> "LDAP://" & Domain Then
            ' let's convert the ADsPath into an LDAP query
            ldap = "LDAP://" & Domain

            entry = New System.DirectoryServices.DirectoryEntry(ldap)
            srch = New DirectoryServices.DirectorySearcher(entry)
        End If

        ' setup the filter
        If AD_Type = "User" Then
            srch.Filter = "(&(&(objectClass=user)(objectCategory=person))(Name=" & Name & "))"
        End If
        If AD_Type = "Group" Then
            srch.Filter = "(&(objectClass=group)(Name=" & Name & "))"
        End If

        ' do the search
        result = srch.FindOne()

        ' trim it up a wee bit
        If Not IsNothing(result) Then
            Return result.Path.Substring(8 + Domain.Length)
        End If
        Return ""
    End Function

    Private Sub bnQuit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bnQuit.Click
        Me.Close()
    End Sub
End Class